@@ -13,9 +13,15 @@ namespace py = pybind11;
 
 using namespace py::literals;
 
+/// Useful for debugging
+void pyPrintPretty(py::handle obj) {
+    auto json = py::module::import("json");
+    py::print(json.attr("dumps")(obj, "indent"_a=py::int_(4)));
+}
+
 #define UNEXPORT __attribute__ ((visibility("hidden")))
 
-constexpr auto TESTED_YTMUSICAPI_VERSION = "0.18.0";
+constexpr auto TESTED_YTMUSICAPI_VERSION = "0.19.4";
 
 struct UNEXPORT YTMusicPrivate {
     py::scoped_interpreter guard {};
@@ -47,7 +53,7 @@ struct UNEXPORT YTMusicPrivate {
     py::object get_ytdl() {
         // lazy initialization
         if (ytdl.is_none()) {
-             ytdl = py::module::import("youtube_dl").attr("YoutubeDL")(py::dict());
+             ytdl = py::module::import("yt_dlp").attr("YoutubeDL")(py::dict());
         }
 
         return ytdl;
@@ -64,7 +70,7 @@ std::optional<T> optional_key(py::handle obj, const char *name) {
         return std::nullopt;
     }
 
-    return obj[name].cast<T>();
+    return obj[name].cast<std::optional<T>>();
 }
 
 meta::Thumbnail extract_thumbnail(py::handle thumbnail) {
@@ -82,28 +88,10 @@ meta::Artist extract_meta_artist(py::handle artist) {
     };
 };
 
-album::Track extract_album_track(py::handle track) {
-    return {
-        track["index"].cast<std::string>(),
-        track["title"].cast<std::string>(),
-        track["artists"].cast<std::string>(),
-        track["videoId"].cast<std::optional<std::string>>(),  // E rated songs don't have a videoId
-        track["lengthMs"].cast<std::optional<std::string>>(), //
-        track["likeStatus"].cast<std::optional<std::string>>()
-    };
-}
-
-video_info::Format extract_format(py::handle format) {
-    return {
-        optional_key<int>(format, "quality"),
-        format["url"].cast<std::string>(),
-        format["vcodec"].cast<std::string>(),
-        format["acodec"].cast<std::string>()
-    };
-}
-
 playlist::Track extract_playlist_track(py::handle track);
 watch::Playlist::Track extract_watch_track(py::handle track);
+album::Track extract_album_track(py::handle track);
+video_info::Format extract_format(py::handle format);
 
 template <typename T>
 inline auto extract_py_list(py::handle obj) {
@@ -135,6 +123,27 @@ inline auto extract_py_list(py::handle obj) {
     return output;
 }
 
+album::Track extract_album_track(py::handle track) {
+    return {
+        track["isExplicit"].cast<bool>(),
+        track["title"].cast<std::string>(),
+        extract_py_list<meta::Artist>(track["artists"]),
+        track["album"].cast<std::optional<std::string>>(),
+        track["videoId"].cast<std::optional<std::string>>(),  // E rated songs don't have a videoId
+        track["duration"].cast<std::optional<std::string>>(), //
+        track["likeStatus"].cast<std::optional<std::string>>()
+    };
+}
+
+video_info::Format extract_format(py::handle format) {
+    return {
+        optional_key<int>(format, "quality"),
+        format["url"].cast<std::string>(),
+        format["vcodec"].cast<std::string>(),
+        format["acodec"].cast<std::string>()
+    };
+}
+
 meta::Album extract_meta_album(py::handle album) {
     return meta::Album {
         album["name"].cast<std::string>(),
@@ -280,7 +289,7 @@ std::optional<search::SearchResultItem> extract_search_result(py::handle result)
             result["title"].cast<std::string>(),
             result["type"].cast<std::string>(),
             extract_py_list<meta::Artist>(result["artists"]),
-            result["year"].cast<std::string>(),
+            result["year"].cast<std::optional<std::string>>(),
             result["isExplicit"].cast<bool>()
         };
     } else if (resultType == "playlist") {
@@ -299,7 +308,8 @@ std::optional<search::SearchResultItem> extract_search_result(py::handle result)
         };
     } else {
         std::cerr << "Warning: Unsupported search result type found" << std::endl;
-        std::cerr << "It's called: " << resultType;
+        std::cerr << "It's called: " << resultType << std::endl;
+        pyPrintPretty(result);
         return std::nullopt;
     }
 }
@@ -324,10 +334,11 @@ YTMusic::~YTMusic() = default;
 std::vector<search::SearchResultItem> YTMusic::search(
         const std::string &query,
         const std::optional<std::string> &filter,
+        const std::optional<std::string> &scope,
         const int limit,
         const bool ignore_spelling) const
 {
-    const auto results = d->get_ytmusic().attr("search")(query, filter, limit, ignore_spelling).cast<py::list>();
+    const auto results = d->get_ytmusic().attr("search")("query"_a=query, "filter"_a=filter, "scope"_a=scope, "limit"_a = limit, "ignore_spelling"_a = ignore_spelling).cast<py::list>();
 
     std::vector<search::SearchResultItem> output;
     for (const auto &result : results) {
@@ -343,7 +354,7 @@ artist::Artist YTMusic::get_artist(const std::string &channel_id) const
 {
     const auto artist = d->get_ytmusic().attr("get_artist")(channel_id);
     return artist::Artist {
-        artist["description"].cast<std::optional<std::string>>(),
+        optional_key<std::string>(artist, "description"),
         artist["views"].cast<std::optional<std::string>>(),
         artist["name"].cast<std::string>(),
         artist["channelId"].cast<std::string>(),
@@ -360,31 +371,28 @@ artist::Artist YTMusic::get_artist(const std::string &channel_id) const
 album::Album YTMusic::get_album(const std::string &browseId) const
 {
     const auto album = d->get_ytmusic().attr("get_album")(browseId);
-
     return {
         album["title"].cast<std::string>(),
-        album["trackCount"].cast<std::string>(),
-        album["durationMs"].cast<std::string>(),
-        album["playlistId"].cast<std::string>(),
-        [&]() -> album::Album::ReleaseDate {
-            const auto date = album["releaseDate"];
-            return {
-                date["year"].cast<int>(),
-                date["month"].cast<int>(),
-                date["day"].cast<int>()
-            };
-        }(),
-        album["description"].cast<std::string>(),
+        album["trackCount"].cast<int>(),
+        album["duration"].cast<std::string>(),
+        album["audioPlaylistId"].cast<std::string>(),
+        album["year"].cast<std::string>(),
+        optional_key<std::string>(album, "description"),
         extract_py_list<meta::Thumbnail>(album["thumbnails"]),
-        extract_py_list<album::Track>(album["tracks"])
+        extract_py_list<album::Track>(album["tracks"]),
+        extract_py_list<meta::Artist>(album["artists"])
     };
 }
 
-song::Song YTMusic::get_song(const std::string &video_id) const
+std::optional<song::Song> YTMusic::get_song(const std::string &video_id) const
 {
     const auto song = d->get_ytmusic().attr("get_song")(video_id);
 
-    return {
+    if (!song.cast<py::dict>().contains("videoId")) {
+        return std::nullopt;
+    }
+
+    return song::Song {
         song["videoId"].cast<std::string>(),
         song["title"].cast<std::string>(),
         song["lengthSeconds"].cast<std::string>(),
